class Git
  require 'fileutils'
  require 'open3'

  attr_reader :repo_path, :branch

  def self.init(git_access, repo_path)
    url = embedded_creds_url(git_access.url, git_access.username, git_access.password)
    repo = Git.new(repo_path)
    raise 'Failed to access repository. Invalid url or credentials.' unless repo.clone(url)
  end

  def initialize(repo_path, branch = 'master')
    @repo_path = repo_path
    @branch = branch
  end

  def persist(add_path = '.', &write_changes)
    retries = 20

    while retries > 0
      retries -= 1
      write_changes.call
      return true unless any_changes?
      return true if add(add_path) && commit && push
      resolve_conflict
    end

    false
  end

  def resolve_conflict
    abort_merge
    hard_reset
    pull
  end

  def clone(url)
    FileUtils.remove_dir(repo_path) rescue nil
    FileUtils.makedirs(repo_path)
    execute("git clone #{url} . && git checkout #{branch}")
  end

  def any_changes?
    execute('git status', false)[:stdout]
        .exclude?('nothing to commit, working directory clean')
  end

  def push
    execute("git push -u origin #{branch}")
  end

  def pull
    execute("git pull origin #{branch}")
  end

  def fetch
    execute('git fetch --all')
  end

  def add(add_path)
    execute("git add #{add_path}")
  end

  def commit(msg = 'ACSmI-contracting: Contract created')
    execute("git config user.email 'ascmi-contracting@cloudbroker.com'")
    execute("git config user.name 'ACSmI Contracting'")
    execute("git commit -m '#{msg}'")
  end

  def abort_merge
    execute('git merge --abort')
  end

  def hard_reset
    execute('git reset --hard')
  end

  class << self

    private

    def embedded_creds_url(url, username, password)
      to_replace = url.include?('@') ? url.scan(/\/\/.*@/).first : '//'
      new_params_str = "//#{CGI.escape(username)}:#{CGI.escape(password)}@"
      url.gsub(to_replace, new_params_str)
    end

  end

  private

  def execute(command, success_flag_only = true)
    logger = Logger.new(File.join('log', 'git.log'))
    Dir.chdir(repo_path) do
      out, err, st = Open3.capture3(command)
      logger.info("BASH COMMAND EXECUTED: #{filter_password command}")
      logger.info("STDOUT: #{filter_password out}")
      logger.info("STDERR: #{filter_password err}")
      logger.info("ST: #{st}")
      success_flag_only ? st.success? : { stdout: out, stderr: err, exist_status: st }
    end
  end

  def filter_password(string)
    regex = /(https|http):\/\/\w+:/i
    protocol_and_username = string[regex]
    return string unless protocol_and_username.present?
    protocol_and_username + '*FILTERED*@' + (string.sub(protocol_and_username, '')).split('@').last
  end
end