/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
package eu.decideh2020.acsmi.backend.services.server.web.rest;

import java.net.URISyntaxException;
import java.time.ZonedDateTime;
import java.util.Optional;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.codahale.metrics.annotation.Timed;

import eu.decideh2020.acsmi.backend.services.server.domain.ServiceAttributeValue;
import eu.decideh2020.acsmi.backend.services.server.repository.ServiceAttributeValueRepository;
import eu.decideh2020.acsmi.backend.services.server.web.rest.util.HeaderUtil;
import io.github.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing ServiceAttributeValue.
 */
@RestController
@RequestMapping("/api")
public class ServiceAttributeValueResource {

    private final Logger log = LoggerFactory.getLogger(ServiceAttributeValueResource.class);

    private static final String ENTITY_NAME = "serviceAttributeValue";
        
    private final ServiceAttributeValueRepository serviceAttributeValueRepository;

    public ServiceAttributeValueResource(ServiceAttributeValueRepository serviceAttributeValueRepository) {
        this.serviceAttributeValueRepository = serviceAttributeValueRepository;
    }
    

    /**
     * POST  /service-attribute-values : Create a new serviceAttributeValue.
     *
     * @param serviceAttributeValue the serviceAttributeValue to create
     * @return the ResponseEntity with status 201 (Created) and with body the new serviceAttributeValue, or with status 400 (Bad Request) if the serviceAttributeValue has already an ID
     * @throws URISyntaxException if the Location URI syntax is incorrect
     */
    @PostMapping("/service-attribute-values")
    @Timed
    @Transactional
    public ResponseEntity<ServiceAttributeValue> createServiceAttributeValue(@Valid @RequestBody ServiceAttributeValue serviceAttributeValue) throws URISyntaxException {
    	log.debug("REST request to save ServiceAttributeValue : {}", serviceAttributeValue);
    	try {
            // Insertar el valor del atributo
            log.debug("createServiceAttributeValue  ::  creating ServiceAttributeValue...");
            serviceAttributeValueRepository.saveAndFlush(serviceAttributeValue);
        	log.debug("createServiceAttributeValue  ::  creating OK");
        	
        	return ResponseEntity.noContent().build();
    	} catch (Exception e) {
    		log.error("createServiceAttributeValue  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceAttributeValueCreateError", "No puede crearse el Valor del atributo"))
                    .body(null);
    	}
    }
    
    /**
     * PUT  /service-attribute-values : Updates an existing serviceAttributeValue.
     *
     * @param serviceAttributeValue the serviceAttributeValue to update
     * @return the ResponseEntity with status 200 (OK) and with body the updated serviceAttributeValue,
     * or with status 400 (Bad Request) if the serviceAttributeValue is not valid,
     * or with status 500 (Internal Server Error) if the serviceAttributeValue couldnt be updated
     * @throws URISyntaxException if the Location URI syntax is incorrect
     */
    @PutMapping("/service-attribute-values")
    @Timed
    @Transactional
    public ResponseEntity<ServiceAttributeValue> updateServiceAttributeValue(@Valid @RequestBody ServiceAttributeValue serviceAttributeValue) throws URISyntaxException {
    	log.debug("REST request to update ServiceAttributeValue : {}", serviceAttributeValue);
        try {
        	// Si la fecha de baja no es nula, se trata de una reactivación
        	if (serviceAttributeValue.getDeleteddate() != null) {
        		log.debug("updateServiceAttributeValue  ::  reactivating ServiceAttributeValue...");
        		serviceAttributeValue.setDeleteddate(null);
        	} else {
        		if (serviceAttributeValue.getId() == null) {
                    return createServiceAttributeValue(serviceAttributeValue);
                }
        	}
        	
        	// Modificar el valor del atributo
        	log.debug("updateServiceAttributeValue  ::  updating ServiceAttributeValue...");
        	serviceAttributeValueRepository.saveAndFlush(serviceAttributeValue);
        	log.debug("updateServiceAttributeValue  ::  updating OK");
        	
        	return ResponseEntity.noContent().build();
        } catch (Exception e) {
        	log.error("updateServiceAttributeValue  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceAttributeValueUpdateError", "No puede actualizarse el Valor del atributo"))
                    .body(null);
        }
    }

    /**
     * GET  /service-attribute-values/:id : get the "id" ServiceAttributeValues.
     *
     * @param id the id of the ServiceAttributeValues to retrieve
     * @return the ResponseEntity with status 200 (OK) and with body the ServiceAttributeValues, or with status 404 (Not Found)
     */
    @GetMapping("/service-attribute-values/{id}")
    @Timed
    public ResponseEntity<ServiceAttributeValue> getServiceAttributeValue(@PathVariable Long id) {
        log.debug("REST request to get getServiceAttributeValue : {}", id);
        ServiceAttributeValue serviceAttributeValue = null;
        try {
        	log.debug("getServiceAttributeValue  ::  getting ServiceClassTypicalAttribute...");
        	serviceAttributeValue = serviceAttributeValueRepository.findOne(id);
        	
        	log.debug("getServiceAttributeValue  ::  getting OK");
            return ResponseUtil.wrapOrNotFound(Optional.ofNullable(serviceAttributeValue));
        } catch (Exception e) {
        	log.error("getServiceAttributeValue  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceAttributeValueGetError", "No puede obtenerse el valor del atributo"))
                    .body(null);
        }
    }
    
    /**
     * DELETE  /service-attribute-values/:id : delete the "id" serviceAttributeValue.
     *
     * @param id the id of the serviceAttributeValue to delete
     * @return the ResponseEntity with status 200 (OK)
     */
    @DeleteMapping("/service-attribute-values/{id}")
    @Timed
    @Transactional
    public ResponseEntity<Void> deleteServiceAttributeValue(@PathVariable Long id, @RequestParam String forcedelete) {
    	log.debug("REST request to delete ServiceAttributeValue : {}", id);
        try {
        	log.debug("deleteServiceAttributeValue  ::  forcedelete=" + forcedelete);
        	
        	// Obtener el valor del atributo
        	ServiceAttributeValue serviceAttributeValue = serviceAttributeValueRepository.findOne(id);
        	
        	if (forcedelete != null && !Boolean.valueOf(forcedelete).booleanValue()) {
        		// Marcar el valor del atributo como borrado
            	if (serviceAttributeValue != null) {
    	        	log.debug("deleteServiceAttributeValue  ::  soft deleting ServiceAttributeValue...");
    	        	
    	        	// Marcar valor del atributo como borrado
    	        	serviceAttributeValue.setDeleteddate(ZonedDateTime.now());
    	        	serviceAttributeValueRepository.saveAndFlush(serviceAttributeValue);
            	}
        	} else {
        		// Eliminar definitivamente el valor del atributo
        		log.debug("deleteServiceAttributeValue  ::  deleting serviceAttributeValue...");
            	serviceAttributeValueRepository.delete(id);
        	}
        	log.debug("deleteServiceAttributeValue  ::  deleting OK");
        	
        	return ResponseEntity.noContent().build();
        } catch (Exception e) {
        	log.error("deleteServiceAttributeValue  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceAttributeValueDeleteError", "El Valor del atributo no puede ser eliminado"))
                    .body(null);
        }
    }
    
    
    /**
     * DELETE  /service-attribute-values/:id : delete the "id" serviceAttributeValue.
     *
     * @param idservice the idService of the serviceAttributeValue to delete
     */
    @Transactional
    @Modifying
    public void deleteByIdService(Long idservice) {
    	log.debug("REST request to delete ServiceAttributeValue by idservice : {}", idservice);
    	try {
    		log.debug("deleteServiceAttributeValueByIdService  ::  deleting serviceAttributeValues...");
    		serviceAttributeValueRepository.deleteByIdService(idservice);
    		log.debug("deleteServiceAttributeValueByIdService  ::  deleting OK");
    	} catch (Exception e) {
        	log.error("deleteServiceAttributeValueByIdService  ::  Exception  ::  " + e.getMessage());
        }
    }
    
    
    /**
     * Update the deleted date with the "id" serviceAttributeValue.
     * @param deletedDate the deletedDate to update
     * @param idservice the idService of the serviceAttributeValue to update
     */
    @Transactional
    @Modifying
    public void updateDeletedDateByIdService(ZonedDateTime deletedDate, Long idservice) {
    	log.debug("REST request to update the deleted date ServiceAttributeValue by idservice : {}", idservice);
    	try {
    		log.debug("updateDeletedDateServiceAttributeValueByIdService  ::  deleting serviceAttributeValues...");
    		serviceAttributeValueRepository.updateDeletedDateByIdService(deletedDate, idservice);
    		log.debug("updateDeletedDateServiceAttributeValueByIdService  ::  deleting OK");
    	} catch (Exception e) {
        	log.error("deleteServiceAttributeValueByIdService  ::  Exception  ::  " + e.getMessage());
        }
    }
}
